@@ -1,5 +1,8 @@
 #!/bin/sh
 #
+# Copyright (c) 2004, 2007 Regents of The University of Michigan.
+# All Rights Reserved.  See COPYRIGHT.
+#
 # Radmind Assistant shell script (rash)
 # Basic client side functions include:
 #
@@ -20,15 +23,19 @@
 # lcreate -l -U
 # -c sha1
 
+KFILE="/var/radmind/client/command.K"
 SERVER="radmind"
 TLSLEVEL="0"
 EDITOR=${EDITOR:-vi}
+USER=${SUDO_USER:-$USER}
+TMPDIR="${TMPDIR:=/tmp}"
 DEFAULTS="/etc/defaults/radmind"
 FSDIFFROOT="."
 FLAG="/var/radmind/client/.RadmindRunning"
 CHECKEDOUT="/var/radmind/client/.CheckedOut"
 MAILDOMAIN=""
-VERSION=`date +%Y%m%d`
+ECHO="/bin/echo"
+VERSION=
 
 PREAPPLY="/var/radmind/preapply"
 POSTAPPLY="/var/radmind/postapply"
@@ -38,16 +45,16 @@ RETRY=10
 
 MKTEMP="/usr/bin/mktemp"
 TEMPFILES=FALSE
-TMPDIR="/tmp/.ra.$$"
-if [ -f "${MKTEMP}" ]; then
-    TMPDIR=`${MKTEMP} -qd /tmp/.ra.$$.XXXXXX`
+RASHTMP="${TMPDIR}/.ra.$$"
+if [ -f "${MKTEMP}"  ]; then
+    RASHTMP=`${MKTEMP} -qd "${TMPDIR}/.ra.$$.XXXXXX"`
     if [ $? -ne 0 ]; then
-	echo "mktemp failed"
+	$ECHO "mktemp failed"
 	exit 1
     fi
 fi
-LTMP="${TMPDIR}/lapply.out"
-FTMP="${TMPDIR}/fsdiff.out"
+LTMP="${RASHTMP}/lapply.out"
+FTMP="${RASHTMP}/fsdiff.out"
 
 # different systems use different default dirs
 if [ ! -f "${DEFAULTS}" ]; then
@@ -58,8 +65,11 @@ if [ ! -f "${DEFAULTS}" ]; then
 fi
 
 Yn() {
-    echo -n "$*" "[Yn] "
+    $ECHO -n "$*" "[Yn] "
     read ans
+    if [ $? -ne 0 ]; then
+	return 0
+    fi
     if [ -z "$ans" -o X"$ans" = Xy -o X"$ans" = XY -o X"$ans" = Xyes ]; then
 	return 1
     fi
@@ -75,39 +85,52 @@ checkedout() {
 }
 
 usage() {
-    echo "Usage:	$0 [ -ctV ] [ -h server ] [ -w authlevel ] { trip | update | create | auto | force | checkout | checkin }" >&2
+    $ECHO "Usage:	$0 [ -ctV ] [ -h server ] [ -w authlevel ] { trip | update | create | auto | force | checkout | checkin } [ /path/or/file ]" >&2
     exit 1
 }
 
 cleanup() {
     if [ "$TEMPFILES" = FALSE ]; then
-	rm -fr "${TMPDIR}"
+	rm -fr "${RASHTMP}"
     fi
 }
 
+cleanup_and_exit() {
+    cleanup
+    exit 1
+}
+
 dopreapply() {
     if [ -d ${PREAPPLY} ]; then
-	for script in ${PREAPPLY}/*; do
-	    ${script} "$1"
-	done
+	SCRIPTS=`find ${PREAPPLY} -perm -u+x \! -type d | sort`
+	if [ "${SCRIPTS}" ]; then
+	    for script in ${SCRIPTS}; do
+		${script} "$1"
+	    done
+	fi
     fi
 }
 
 dopostapply() {
     if [ -d ${POSTAPPLY} ]; then
-	for script in ${POSTAPPLY}/*; do
-	    ${script} "$1"
-	done
+	SCRIPTS=`find ${POSTAPPLY} -perm -u+x \! -type d | sort`
+	if [ "${SCRIPTS}" ]; then
+	    for script in ${SCRIPTS}; do
+		${script} "$1"
+	    done
+	fi
     fi
 }
 
 update() {
     opt="$1"
     kopt=
+    apply=ask
+    can_edit=no
 
     checkedout
     if [ $? -eq 1 ]; then
-	echo "Checked out by ${OWNER}"
+	$ECHO "Checked out by ${OWNER}"
 	if [ x"$opt" = x"interactive" -a x"$USER" = x"$OWNER" ]; then
 	    Yn "Continue with update?"
 	    if [ $? -eq 0 ]; then
@@ -124,19 +147,21 @@ update() {
 
     ktcheck ${kopt} -w ${TLSLEVEL} -h ${SERVER} -c sha1
     case "$?" in
-    0)  if [ x"$opt" = x"hook" -a ! -f "${FLAG}" ]; then
+    0)  
+	if [ x"$opt" = x"hook" -a ! -f "${FLAG}" ]; then
 	    cleanup
 	    exit 0
 	fi
 	;;
 
-    1)	if [ x"$opt" = x"interactive" ]; then
+    1)
+	if [ x"$opt" = x"interactive" ]; then
 	    Yn "Update command file and/or transcripts?"
 	    if [ $? -eq 1 ]; then
 		ktcheck -w ${TLSLEVEL} -h ${SERVER} -c sha1
 		RC=$?
 		if [ $RC -ne 1 ]; then
-		    echo Nothing to update
+		    $ECHO Nothing to update
 		    cleanup
 		    exit $RC
 		fi
@@ -144,30 +169,33 @@ update() {
 	fi
 	;;
 
-    *)	cleanup
+    *)	
+	cleanup
     	exit $?
 	;;
     esac
 
-    fsdiff -A -% ${CHECKSUM} -o ${FTMP} ${FSDIFFROOT}
+    fsdiff -A ${CASE} ${FPROGRESS} ${CHECKSUM} -o ${FTMP} ${FSDIFFROOT}
     if [ $? -ne 0 ]; then
 	cleanup
 	exit 1
     fi
 
     if [ ! -s ${FTMP} ]; then
-	echo Nothing to apply.
+	$ECHO Nothing to apply.
 	cleanup
 	exit 0
     fi
     if [ x"$opt" = x"interactive" ]; then
-	Yn "Edit difference transcript?"
-	if [ $? -eq 1 ]; then
-	    ${EDITOR} ${FTMP}
+	cat ${FTMP}
+	infocmp >/dev/null 2>&1
+	if [ $? -eq 0 ]; then
+	    can_edit=yes
 	fi
     fi
     
-    if [ x"$opt" = x"interactive" -a -d "${PREAPPLY}" ]; then
+    if [ x"$opt" = x"interactive" -a -d "${PREAPPLY}" \
+		-a ! -z "`ls ${PREAPPLY} 2>/dev/null`" ]; then
 	Yn "Run pre-apply scripts on difference transcript?"
         if [ $? -eq 1 ]; then
             dopreapply ${FTMP}
@@ -175,35 +203,66 @@ update() {
     elif [ x"$opt" != x"interactive" ]; then
 	dopreapply ${FTMP}
     fi
-    if [ x"$opt" = x"interactive" ]; then
-	Yn "Apply difference transcript?"
-	if [ $? -ne 1 ]; then
-	    cleanup
-	    exit 0
-	fi
+    if [ x"${opt}" = x"interactive" ]; then
+	while [ 1 ]; do
+	    if [ x"${can_edit}" = x"yes" ]; then
+		$ECHO -n "(e)dit difference transcript, "
+	    fi
+	    $ECHO -n "(a)pply or (c)ancel? "
+
+	    read ans
+	    if [ $? -ne 0 ]; then
+		cleanup_and_exit
+	    fi
+
+	    case "${ans}" in
+	    a|A)
+		break
+		;;
+
+	    c|C)
+		$ECHO
+		$ECHO Update cancelled
+		cleanup
+		exit 0
+		;;
+
+	    e|E)
+		if [ x"${can_edit}" = x"yes" ]; then
+		    ${EDITOR} ${FTMP}
+		fi
+		;;
+
+	    *)
+		;;
+
+	    esac
+	done
     fi
-    lapply ${PROGRESS} -w ${TLSLEVEL} -h ${SERVER} ${CHECKSUM} ${FTMP}
+		
+    lapply ${CASE} ${PROGRESS} -w ${TLSLEVEL} -h ${SERVER} ${CHECKSUM} ${FTMP}
     case "$?" in
     0)	;;
 
     *)  if [ x"$opt" = x"hook" ]; then
-	    echo -n "Applying changes failed, trying again "
-	    echo "in ${RETRY} seconds..."
+	    $ECHO -n "Applying changes failed, trying again "
+	    $ECHO "in ${RETRY} seconds..."
 	    sleep ${RETRY}
 	    RETRY=${RETRY}0
 
-	    echo %OPENDRAWER
-	    echo %BEGINPOLE
+	    $ECHO %OPENDRAWER
+	    $ECHO %BEGINPOLE
     	else 
 	    cleanup
 	fi
 	return 1
 	;;
     esac
-    if [ x"$opt" = x"interactive" -a -d "${POSTAPPLY}" ]; then
+    if [ x"$opt" = x"interactive" -a -d "${POSTAPPLY}" \
+		-a ! -z "`ls ${POSTAPPLY} 2>/dev/null`" ]; then
 	Yn "Run post-apply scripts on difference transcript?"
         if [ $? -eq 1 ]; then
-            dopostapply ${FMTP}
+            dopostapply ${FTMP}
         fi
     elif [ x"$opt" != x"interactive" ]; then
 	dopostapply ${FTMP}
@@ -219,9 +278,13 @@ if [ -f "${DEFAULTS}" ]; then
     . "${DEFAULTS}"
 fi
 
-while getopts %ch:ltVw: opt; do
+while getopts %ch:Ilqr:tU:Vw: opt; do
     case $opt in
     %)  PROGRESS="-%"
+	FPROGRESS="-%"
+	;;
+
+    q)  PROGRESS="-q"
 	;;
 
     c)	CHECKSUM="-csha1"
@@ -230,13 +293,23 @@ while getopts %ch:ltVw: opt; do
     h)	SERVER="$OPTARG"
     	;;
 
+    I)	CASE="-I"
+	;;
+
     l)  USERAUTH="-l"
 	;;
 
+    r)  FSDIFFROOT="$OPTARG"
+	;;
+
     t)	TEMPFILES="TRUE"
     	;;
 
-    V)	echo ${VERSION}
+    U)	USER="$OPTARG"
+	USERNAME="$OPTARG"
+    	;;
+
+    V)	$ECHO ${VERSION}
 	exit 0
 	;;
 
@@ -249,49 +322,60 @@ while getopts %ch:ltVw: opt; do
 done
 shift `expr $OPTIND - 1`
 
-if [ $# -ne 1 ]; then
-	usage
+if [ $# -eq 2 ]; then
+    FSDIFFROOT=$2
+elif [ $# -ne 1 ]; then
+    usage
 fi
 
 cd /
 
-if [ ! -d ${TMPDIR} ]; then
-    mkdir -m 700 ${TMPDIR} 
+if [ ! -d "${RASHTMP}" ]; then
+    mkdir -m 700 "${RASHTMP}"
     if [ $? -ne 0 ]; then
-        echo "Cannot create temporary directory $TMPDIR" 
+        $ECHO "Cannot create temporary directory $RASHTMP" 
 	exit 1
     fi
 fi
 
 # Trap meaningful signals
-trap cleanup HUP INT PIPE QUIT TERM TRAP XCPU XFSZ
+trap cleanup_and_exit HUP INT PIPE QUIT TERM TRAP XCPU XFSZ
 
 case "$1" in
 checkout)
+    if [ ${USER} = root ]; then
+	$ECHO -n "Username? [root] "
+	read ans
+	USER=${ans:-root}
+    fi
     checkedout
     if [ $? -eq 1 ]; then
+	$ECHO "Already checked out by ${OWNER}"
 	if [ x${OWNER} = x${USER} ]; then
-	    echo "Already checked out"
 	    exit 1
 	fi
-	echo "Already checked out by ${OWNER}"
 	Yn "Force checkout?"
 	if [ $? -eq 0 ]; then
 	    exit 1
 	fi
-	echo ${USER} has removed your checkout on `hostname` | mail -s `hostname`": Checkout broken" ${OWNER}@${MAILDOMAIN:-`hostname`}
+	$ECHO ${USER} has removed your checkout on `hostname` | mail -s `hostname`": Checkout broken" ${OWNER}@${MAILDOMAIN:-`hostname`}
     fi
-    echo ${USER} > ${CHECKEDOUT}
+    $ECHO ${USER} > ${CHECKEDOUT}
     ;;
 
 checkin)
     checkedout
     if [ $? -eq 0 ]; then
-	echo "Not checked out"
+	$ECHO "Not checked out"
 	exit 1
     fi
+    if [ ${USER} = root ]; then
+	$ECHO -n "Username? [root] "
+	read ans
+	USER=${ans:-root}
+    fi
     if [ x${OWNER} != x${USER} ]; then
-	echo "Currently checked out by ${OWNER}"
+	$ECHO "Currently checked out by ${OWNER}"
 	exit 1
     fi
     rm ${CHECKEDOUT}
@@ -313,7 +397,7 @@ create)
 	    ktcheck -w ${TLSLEVEL} -h ${SERVER} -c sha1
 	    RC=$?
 	    if [ $RC -ne 1 ]; then
-		echo Nothing to update
+		$ECHO Nothing to update
 		cleanup
 		exit $RC
 	    fi
@@ -324,19 +408,19 @@ create)
    	exit $?
     	;;
     esac
-    echo -n "Enter new transcript name [`hostname | cut -d. -f1`-`date +%Y%m%d`-${USER}.T]: "
+    $ECHO -n "Enter new transcript name [`hostname | cut -d. -f1`-`date +%Y%m%d`-${USER}.T]: "
     read TNAME
     if [ -z "${TNAME}" ]; then
 	TNAME=`hostname | cut -d. -f1`-`date +%Y%m%d`-${USER}.T
     fi
-    FTMP="${TMPDIR}/${TNAME}"
-    fsdiff -C -% ${CHECKSUM} -o ${FTMP} ${FSDIFFROOT}
+    FTMP="${RASHTMP}/${TNAME}"
+    fsdiff -C ${CASE} ${FPROGRESS} ${CHECKSUM} -o ${FTMP} ${FSDIFFROOT}
     if [ $? -ne 0 ]; then
 	cleanup
 	exit 1;
     fi
     if [ ! -s ${FTMP} ]; then
-	echo Nothing to create.
+	$ECHO Nothing to create.
 	cleanup
 	exit 1
     fi
@@ -347,8 +431,10 @@ create)
     Yn "Store loadset ${TNAME}?"
     if [ $? -eq 1 ]; then
 	if [ -n "${USERAUTH}" ]; then
-	    echo -n "username: "
-	    read USERNAME
+	    if [ -z "${USERNAME}" ]; then
+		$ECHO -n "username: "
+		read USERNAME
+	    fi
 	    USERNAME="-U ${USERNAME}"
 	fi
 	lcreate ${PROGRESS} -w ${TLSLEVEL} ${USERAUTH} ${USERNAME} \
@@ -362,13 +448,18 @@ create)
     ;;
 
 trip)
+    if [ ! -f ${KFILE} ]; then
+	$ECHO Command file missing, skipping tripwire.
+	cleanup
+	exit 1
+    fi
     # Since trip does not modify the system, no need for checkedout
     ktcheck -w ${TLSLEVEL} -h ${SERVER} -qn -c sha1
     case "$?" in
     0)
 	;;
     1)
-	echo Command file and/or transcripts are out of date.
+	$ECHO Command file and/or transcripts are out of date.
 	;;
     *)
 	cleanup
@@ -376,13 +467,13 @@ trip)
 	;;
     esac
 
-    fsdiff -C ${CHECKSUM} -o ${FTMP} ${FSDIFFROOT}
+    fsdiff -C ${CASE} ${CHECKSUM} -o ${FTMP} ${FSDIFFROOT}
     if [ $? -ne 0 ]; then
 	cleanup
 	exit 1
     fi
     if [ -s ${FTMP} ]; then
-	echo Trip failure: `hostname`
+	$ECHO Trip failure: `hostname`
 	cat ${FTMP}
 	cleanup
 	exit 0
@@ -392,17 +483,17 @@ trip)
 auto)
     checkedout
     if [ $? -eq 1 ]; then
-	echo "Checked out by ${OWNER}"
+	$ECHO "Checked out by ${OWNER}"
 	exit 1
     fi
-    fsdiff -C ${CHECKSUM} -o ${FTMP} ${FSDIFFROOT}
+    fsdiff -C ${CASE} ${CHECKSUM} -o ${FTMP} ${FSDIFFROOT}
     if [ $? -ne 0 ]; then
-	echo Auto failure: `hostname` fsdiff
+	$ECHO Auto failure: `hostname` fsdiff
 	cleanup
 	exit 1
     fi
     if [ -s ${FTMP} ]; then
-	echo Auto failure: `hostname` trip
+	$ECHO Auto failure: `hostname` trip
 	cat ${FTMP}
 	cleanup
 	exit 1
@@ -412,37 +503,43 @@ auto)
     ktcheck -w ${TLSLEVEL} -h ${SERVER} -q -c sha1
     if [ $? -eq 1 ]; then
 	while true; do
-	    fsdiff -A ${CHECKSUM} -o ${FTMP} ${FSDIFFROOT}
+	    fsdiff -A ${CASE} ${CHECKSUM} -o ${FTMP} ${FSDIFFROOT}
 	    if [ $? -ne 0 ]; then
-		echo Auto failure: `hostname`: fsdiff
+		$ECHO Auto failure: `hostname`: fsdiff
 		cleanup
 		exit 1
 	    fi
+	    dopreapply ${FTMP}
 	    if [ -s ${FTMP} ]; then
-		lapply -w ${TLSLEVEL} -h ${SERVER} -q ${CHECKSUM} \
-			${FTMP} 2>&1 > ${LTMP}
+		lapply -w ${TLSLEVEL} ${CASE} ${PROGRESS} -h ${SERVER} \
+			-q ${CHECKSUM} ${FTMP} 2>&1 > ${LTMP}
 		case $? in
 		0)
-		    echo Auto update: `hostname`
+		    $ECHO Auto update: `hostname`
 		    cat ${FTMP}
+		    dopostapply ${FTMP}
 		    cleanup
 		    break
 		    ;;
 
 		*)
 		    if [ ${RETRY} -gt 10000 ]; then
-			echo Auto failure: `hostname`
+			$ECHO Auto failure: `hostname`
 			cat ${LTMP}
 			cleanup
 			exit 1
 		    fi
-		    echo Auto failure: `hostname` retrying
+		    $ECHO Auto failure: `hostname` retrying
 		    cat ${LTMP}
 		    sleep ${RETRY}
 		    RETRY=${RETRY}0
 		    ktcheck -w ${TLSLEVEL} -h ${SERVER} -q -c sha1
 		    ;;
 		esac
+	    else
+		$ECHO Nothing to apply.
+		cleanup
+		exit 0
 	    fi
 	done
     fi
@@ -451,7 +548,7 @@ auto)
 force)
     checkedout
     if [ $? -eq 1 ]; then
-	echo "Checked out by ${OWNER}"
+	$ECHO "Checked out by ${OWNER}"
 	exit 1
     fi
     ktcheck -w ${TLSLEVEL} -h ${SERVER} -c sha1
@@ -464,20 +561,20 @@ force)
 	;;
     esac
 
-    fsdiff -A -% ${CHECKSUM} -o ${FTMP} ${FSDIFFROOT}
+    fsdiff -A ${CASE} ${FPROGRESS} ${CHECKSUM} -o ${FTMP} ${FSDIFFROOT}
     if [ $? -ne 0 ]; then
 	cleanup
 	exit 1
     fi
 
     if [ ! -s ${FTMP} ]; then
-	echo Nothing to apply.
+	$ECHO Nothing to apply.
 	cleanup
 	exit 0
     fi
     
     dopreapply ${FTMP}
-    lapply ${PROGRESS} -w ${TLSLEVEL} -h ${SERVER} ${CHECKSUM} ${FTMP}
+    lapply ${CASE} ${PROGRESS} -w ${TLSLEVEL} -h ${SERVER} ${CHECKSUM} ${FTMP}
     case "$?" in
     0)	;;
 
